from datetime import datetime
from multiprocessing import Queue
from . import INFO, WARNING, ERROR


class LogCat(object):
    def __init__(self, log_queue: Queue, category: str):
        self.queue = log_queue

        self.category = category

    def _write(self, level: int, function: str, message: str) -> None:
        time = datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
        self.queue.put((level, f"{self.level_string(level)} | {time} - {self.category.upper()}.{function.upper()}: {message}"))

    def info(self, function: str, message: str) -> None:
        self._write(INFO, function, message)

    def warning(self, function: str, message: str) -> None:
        self._write(WARNING, function, message)

    def error(self, function: str, message: str) -> None:
        self._write(ERROR, function, message)

    @staticmethod
    def level_string(level: int) -> str:
        if level == INFO:
            return "INFO"
        elif level == WARNING:
            return "WARN"
        elif level == ERROR:
            return "ERR "
