from pathlib import Path


audio_extensions = ['.flac']
art_extensions = ['.jpg', '.jpeg', '.png']

class File:
    def __init__(self, location: Path):
        self.path = location

    def __str__(self):
        return str(self.path)

    @property
    def is_audio(self):
        return self.extension in audio_extensions

    @property
    def is_art(self):
        return self.extension in art_extensions

    @property
    def art_named_correctly(self):
        return self.filename == "cover.jpg"

    @property
    def filename(self) -> str:
        return self.path.name

    @property
    def stem(self) -> str:
        return self.path.stem

    @property
    def extension(self) -> str:
        return self.path.suffix

    @property
    def artist(self) -> Path:
        return self.path.parent.parent

    def artist_out(self, root: Path) -> Path:
        return root / self.artist.parts[-1]

    @property
    def album(self) -> Path:
        return self.path.parent

    def album_out(self, root: Path) -> Path:
        return root /self.artist_out(root) / self.album.parts[-1]

    def artist_output(self, root: Path) -> Path:
        return root / self.artist.parts[-1]

    def album_output(self, root: Path) -> Path:
        return root / self.artist.parts[-1] / self.album.parts[-1]

    def output_file(self, root: Path, extension: str) -> Path:
        return root / self.artist.parts[-1] / self.album.parts[-1] / f"{self.stem}.{extension}"
